/*
 * temp_objects.c
 *
 *  Created on: Mar 6, 2013
 *      Author: msikstrom
 */

#include <naemon/naemon.h>
#include <stdlib.h>
#include <string.h>

#include "temp_objects.h"

/* Create a prototype for a given object */
#define OBJ_PROTOTYPE(_OBJ) \
	static struct _OBJ *temp_create_ ## _OBJ(struct kvvec *kvv, struct _OBJ *obj);

/* Start of a kvvec -> object converter function */
#define INIT_OBJ_DEF(_OBJ)										  \
	static struct _OBJ *temp_create_ ## _OBJ(struct kvvec *kvv, struct _OBJ *obj) {		\
		int i;               \
		char *key;           \
		char *value;         \
		(void)key;           \
		(void)value;         \
		if( obj == NULL ) return NULL; \
		for (i = 0; i < kvv->kv_pairs; i++) { \
			key = kvv->kv[i].key; \
			value = kvv->kv[i].value;


/* Define a parameter in an object */
#define GET_ARG_ADD(_OBJ, _VAR, _FNC) \
		if( 0==strcmp(#_OBJ "." #_VAR, key) ) { \
			obj->_VAR = _FNC(value, obj->_VAR); \
		}

/* Define a parameter in an object */
#define GET_ARG(_OBJ, _VAR, _FNC) \
		if( 0==strcmp(#_OBJ "." #_VAR, key) ) { \
			obj->_VAR = _FNC(value);            \
		}

/* Finish a kvvec -> object converter function */
#define FINISH_OBJ_DEF \
		} \
		return obj;      \
	}

OBJ_PROTOTYPE( contact )
OBJ_PROTOTYPE( contactgroup )
OBJ_PROTOTYPE( host )
OBJ_PROTOTYPE( service )
OBJ_PROTOTYPE( hostgroup )
OBJ_PROTOTYPE( servicegroup )
OBJ_PROTOTYPE( command )

static customvariablesmember *unpack_custom_var( char *customvars, customvariablesmember *prev ) {
	customvariablesmember *cvar;
	char *delim = strchr( customvars, '=' );
	int name_len;

	if( delim == NULL ) {
		/* Could not parse, skip */
		return prev;
	}
	name_len = delim - customvars;

	cvar = calloc(1,sizeof(customvariablesmember));
	cvar->next = prev;
	cvar->variable_name = strndup(customvars, name_len); /* Only copy name */
	if( !cvar->variable_name ) {
		free( cvar );
		/* FIXME: error handling */
		return prev;
	}
	cvar->variable_value = strdup(delim + 1);
	if( !cvar->variable_value ) {
		free( cvar->variable_name );
		free( cvar );
		/* FIXME: error handling */
		return prev;
	}
	return cvar;
}

INIT_OBJ_DEF(contact)
	GET_ARG( contact, alias,                                strdup)
	GET_ARG( contact, email,                                strdup)
	GET_ARG( contact, pager,                                strdup)
//char    *address[MAX_CONTACT_ADDRESSES];
//struct commandsmember *host_notification_commands;
//struct commandsmember *service_notification_commands;
	GET_ARG( contact, host_notification_options,            atoi ) /* unsigned int */
	GET_ARG( contact, service_notification_options,         atoi ) /* unsigned int */
	GET_ARG( contact, minimum_value,                        atoi ) /* unsigned int */
	GET_ARG( contact, host_notification_period,             )
	GET_ARG( contact, service_notification_period,          )
	GET_ARG( contact, host_notifications_enabled,           atoi )
	GET_ARG( contact, service_notifications_enabled,        atoi )
	GET_ARG( contact, can_submit_commands,                  atoi )
	GET_ARG( contact, retain_status_information,            atoi )
	GET_ARG( contact, retain_nonstatus_information,         atoi )
	GET_ARG_ADD( host, custom_variables,                    unpack_custom_var )
FINISH_OBJ_DEF

INIT_OBJ_DEF(contactgroup)
FINISH_OBJ_DEF

INIT_OBJ_DEF(host)
	GET_ARG( host, display_name,                            strdup)
	GET_ARG( host, alias,                                   strdup)
	GET_ARG( host, address,                                 strdup)
	GET_ARG( host, check_command,                           strdup)
	GET_ARG( host, initial_state,                           atoi )
	GET_ARG( host, check_interval,                          atof )
	GET_ARG( host, retry_interval,                          atof )
	GET_ARG( host, max_attempts,                            atoi )
	GET_ARG( host, event_handler,                           strdup)
//struct contactgroupsmember *contact_groups;
//struct contactsmember *contacts;
	GET_ARG( host, notification_interval,                   atof )
	GET_ARG( host, first_notification_delay,                atof )
	GET_ARG( host, notification_options,                    atoi ) /* unsigned int */
	GET_ARG( host, hourly_value,                            atoi ) /* unsigned int */
	GET_ARG( host, notification_period,                     strdup)
	GET_ARG( host, check_period,                            strdup)
	GET_ARG( host, flap_detection_enabled,                  atoi )
	GET_ARG( host, low_flap_threshold,                      atof )
	GET_ARG( host, high_flap_threshold,                     atof )
	GET_ARG( host, flap_detection_options,                  atoi )
	GET_ARG( host, stalking_options,                        atoi ) /* unsigned int */
	GET_ARG( host, check_freshness,                         atoi )
	GET_ARG( host, freshness_threshold,                     atoi )
	GET_ARG( host, process_performance_data,                atoi )
	GET_ARG( host, checks_enabled,                          atoi )
//const char *check_source;
	GET_ARG( host, accept_passive_checks,                   atoi )
	GET_ARG( host, event_handler_enabled,                   atoi )
	GET_ARG( host, retain_status_information,               atoi )
	GET_ARG( host, retain_nonstatus_information,            atoi )
	GET_ARG( host, obsess,                                  atoi )
	GET_ARG( host, notes,                                   strdup)
	GET_ARG( host, notes_url,                               strdup)
	GET_ARG( host, action_url,                              strdup)
	GET_ARG( host, icon_image,                              strdup)
	GET_ARG( host, icon_image_alt,                          strdup)
	GET_ARG( host, statusmap_image,                         strdup)
	GET_ARG( host, vrml_image,                              strdup)
	GET_ARG( host, have_2d_coords,                          atoi )
	GET_ARG( host, x_2d,                                    atoi )
	GET_ARG( host, y_2d,                                    atoi )
	GET_ARG( host, have_3d_coords,                          atoi )
	GET_ARG( host, x_3d,                                    atof )
	GET_ARG( host, y_3d,                                    atof )
	GET_ARG( host, z_3d,                                    atof )
	GET_ARG_ADD( host, custom_variables,                    unpack_custom_var )
	GET_ARG( host, problem_has_been_acknowledged,           atoi )
	GET_ARG( host, acknowledgement_type,                    atoi )
	GET_ARG( host, check_type,                              atoi )
	GET_ARG( host, current_state,                           atoi )
	GET_ARG( host, last_state,                              atoi )
	GET_ARG( host, last_hard_state,                         atoi )
	GET_ARG( host, plugin_output,                           strdup)
	GET_ARG( host, long_plugin_output,                      strdup)
	GET_ARG( host, perf_data,                               strdup)
	GET_ARG( host, state_type,                              atoi )
	GET_ARG( host, current_attempt,                         atoi )
	GET_ARG( host, current_event_id,                        atol ) /* unsigned long */
	GET_ARG( host, last_event_id,                           atol ) /* unsigned long */
	GET_ARG( host, current_problem_id,                      atol ) /* unsigned long */
	GET_ARG( host, last_problem_id,                         atol ) /* unsigned long */
	GET_ARG( host, latency,                                 atof )
	GET_ARG( host, execution_time,                          atof )
	GET_ARG( host, is_executing,                            atoi )
	GET_ARG( host, check_options,                           atoi )
	GET_ARG( host, notifications_enabled,                   atoi )
	GET_ARG( host, last_notification,                       atol ) /* time_t */
	GET_ARG( host, next_notification,                       atol ) /* time_t */
	GET_ARG( host, next_check,                              atol ) /* time_t */
	GET_ARG( host, last_check,                              atol ) /* time_t */
	GET_ARG( host, last_state_change,                       atol ) /* time_t */
	GET_ARG( host, last_hard_state_change,                  atol ) /* time_t */
	GET_ARG( host, last_time_up,                            atol ) /* time_t */
	GET_ARG( host, last_time_down,                          atol ) /* time_t */
	GET_ARG( host, last_time_unreachable,                   atol ) /* time_t */
	GET_ARG( host, has_been_checked,                        atoi )
	GET_ARG( host, is_being_freshened,                      atoi )
	GET_ARG( host, notified_on,                             atoi )
	GET_ARG( host, current_notification_number,             atoi )
	GET_ARG( host, no_more_notifications,                   atoi )
	GET_ARG( host, current_notification_id,                 atol ) /* unsigned long */
	GET_ARG( host, check_flapping_recovery_notification,    atoi )
	GET_ARG( host, scheduled_downtime_depth,                atoi )
	GET_ARG( host, pending_flex_downtime,                   atoi )
//int     state_history[MAX_STATE_HISTORY_ENTRIES];
//int     state_history_index;
	GET_ARG( host, last_state_history_update,               atol ) /* time_t */
	GET_ARG( host, is_flapping,                             atoi )
	GET_ARG( host, flapping_comment_id,                     atol ) /* unsigned long */
	GET_ARG( host, percent_state_change,                    atof )
	GET_ARG( host, modified_attributes,                     atol ) /* unsigned long */
//struct command *event_handler_ptr;
//struct command *check_command_ptr;
//struct timeperiod *check_period_ptr;
//struct timeperiod *notification_period_ptr;
//struct objectlist *hostgroups_ptr;
//struct objectlist *exec_deps, *notify_deps;
//struct objectlist *escalation_list;*/
FINISH_OBJ_DEF

INIT_OBJ_DEF(service)
	GET_ARG( service, display_name,                         strdup)
//struct servicesmember *parents;
//struct servicesmember *children;
	GET_ARG( service, event_handler,                        strdup)
	GET_ARG( service, initial_state,                        atoi )
	GET_ARG( service, check_interval,                       atof )
	GET_ARG( service, retry_interval,                       atof )
	GET_ARG( service, max_attempts,                         atoi )
//struct contactgroupsmember *contact_groups;
//struct contactsmember *contacts;
	GET_ARG( service, notification_interval,                atof )
	GET_ARG( service, first_notification_delay,             atof )
	GET_ARG( service, notification_options,                 atoi ) /* unsigned int */
	GET_ARG( service, stalking_options,                     atoi ) /* unsigned int */
	GET_ARG( service, hourly_value,                         atoi ) /* unsigned int */
	GET_ARG( service, is_volatile,                          atoi ) /* unsigned int */
	GET_ARG( service, notification_period,                  strdup)
	GET_ARG( service, check_period,                         strdup)
	GET_ARG( service, flap_detection_enabled,               atoi )
	GET_ARG( service, low_flap_threshold,                   atof )
	GET_ARG( service, high_flap_threshold,                  atof )
	GET_ARG( service, flap_detection_options,               atoi ) /* unsigned int */
	GET_ARG( service, process_performance_data,             atoi )
	GET_ARG( service, check_freshness,                      atoi )
	GET_ARG( service, freshness_threshold,                  atoi )
	GET_ARG( service, accept_passive_checks,                atoi )
	GET_ARG( service, event_handler_enabled,                atoi )
	GET_ARG( service, checks_enabled,                       atoi )
	GET_ARG( service, check_source,                         strdup)
	GET_ARG( service, retain_status_information,            atoi )
	GET_ARG( service, retain_nonstatus_information,         atoi )
	GET_ARG( service, notifications_enabled,                atoi )
	GET_ARG( service, obsess,                               atoi )
	GET_ARG( service, notes,                                strdup)
	GET_ARG( service, notes_url,                            strdup)
	GET_ARG( service, action_url,                           strdup)
	GET_ARG( service, icon_image,                           strdup)
	GET_ARG( service, icon_image_alt,                       strdup)
	GET_ARG_ADD( service, custom_variables,                 unpack_custom_var )
//#ifdef NSCORE
	GET_ARG( service, problem_has_been_acknowledged,        atoi )
	GET_ARG( service, acknowledgement_type,                 atoi )
	GET_ARG( service, host_problem_at_last_check,           atoi )
	GET_ARG( service, check_type,                           atoi )
	GET_ARG( service, current_state,                        atoi )
	GET_ARG( service, last_state,                           atoi )
	GET_ARG( service, last_hard_state,                      atoi )
	GET_ARG( service, plugin_output,                        strdup)
	GET_ARG( service, long_plugin_output,                   strdup)
	GET_ARG( service, perf_data,                            strdup)
	GET_ARG( service, state_type,                           atoi )
	GET_ARG( service, next_check,                           atol ) /* time_t */
	GET_ARG( service, last_check,                           atol ) /* time_t */
	GET_ARG( service, current_attempt,                      atoi )
	GET_ARG( service, current_event_id,                     atol ) /* unsigned long */
	GET_ARG( service, last_event_id,                        atol ) /* unsigned long */
	GET_ARG( service, current_problem_id,                   atol ) /* unsigned long */
	GET_ARG( service, last_problem_id,                      atol ) /* unsigned long */
	GET_ARG( service, last_notification,                    atol ) /* time_t */
	GET_ARG( service, next_notification,                    atol ) /* time_t */
	GET_ARG( service, no_more_notifications,                atoi )
	GET_ARG( service, check_flapping_recovery_notification, atoi )
	GET_ARG( service, last_state_change,                    atol ) /* time_t */
	GET_ARG( service, last_hard_state_change,               atol ) /* time_t */
	GET_ARG( service, last_time_ok,                         atol ) /* time_t */
	GET_ARG( service, last_time_warning,                    atol ) /* time_t */
	GET_ARG( service, last_time_unknown,                    atol ) /* time_t */
	GET_ARG( service, last_time_critical,                   atol ) /* time_t */
	GET_ARG( service, has_been_checked,                     atoi )
	GET_ARG( service, is_being_freshened,                   atoi )
	GET_ARG( service, notified_on,                          atoi ) /* unsigned int */
	GET_ARG( service, current_notification_number,          atoi )
	GET_ARG( service, current_notification_id,              atol ) /* unsigned long */
	GET_ARG( service, latency,                              atof )
	GET_ARG( service, execution_time,                       atof )
	GET_ARG( service, is_executing,                         atoi )
	GET_ARG( service, check_options,                        atoi )
	GET_ARG( service, scheduled_downtime_depth,             atoi )
	GET_ARG( service, pending_flex_downtime,                atoi )
//int     state_history[MAX_STATE_HISTORY_ENTRIES];    /* flap detection */
//int     state_history_index;
	GET_ARG( service, is_flapping,                          atoi )
	GET_ARG( service, flapping_comment_id,                  atol ) /* unsigned long */
	GET_ARG( service, percent_state_change,                 atof )
	GET_ARG( service, modified_attributes,                  atol )

//struct host *host_ptr;
//struct command *event_handler_ptr;
	GET_ARG( service, event_handler_args,                   strdup)
//struct command *check_command_ptr;
//struct timeperiod *check_period_ptr;
//struct timeperiod *notification_period_ptr;
//struct objectlist *servicegroups_ptr;
//#endif
//struct objectlist *exec_deps, *notify_deps;
//struct objectlist *escalation_list;
//struct service *next;
//struct timed_event *next_check_event;
FINISH_OBJ_DEF

INIT_OBJ_DEF(hostgroup)
FINISH_OBJ_DEF

INIT_OBJ_DEF(servicegroup)
FINISH_OBJ_DEF

INIT_OBJ_DEF(command)
FINISH_OBJ_DEF

struct temp_objects *build_temp_objects( struct kvvec *kvv ) {
	struct temp_objects *objs = calloc(1,sizeof(struct temp_objects));
	char *tmp;

	if( objs == NULL )
		return NULL;

	tmp = kvvec_fetch_str_str(kvv, "contact.name");
	if (tmp) {
		contact *cntct = create_contact(tmp);
		objs->ctc = temp_create_contact(kvv, cntct);
	}
	tmp = kvvec_fetch_str_str(kvv, "contactgroup.group_name");
	if (tmp) {
		contactgroup *cg = create_contactgroup(tmp, kvvec_fetch_str_str(kvv, "contactgroup.alias"));
		objs->cg = temp_create_contactgroup(kvv, cg);
	}
	tmp = kvvec_fetch_str_str(kvv, "host.name");
	host *hst = NULL;
	if (tmp) {
		hst = create_host(kvvec_fetch_str_str(kvv, "host.name"));
		objs->hst = temp_create_host(kvv, hst);
	}
	if (hst) {
		tmp = kvvec_fetch_str_str(kvv, "service.description");
		if (tmp) {
			service *svc = create_service(objs->hst, tmp);
			objs->svc = temp_create_service(kvv, svc);
		}
	}

	tmp = kvvec_fetch_str_str(kvv, "hostgroup.group_name");
	if (tmp) {
		hostgroup *hg = create_hostgroup(
			tmp,
			kvvec_fetch_str_str(kvv, "hostgroup.alias"),
			kvvec_fetch_str_str(kvv, "hostgroup.notes"),
			kvvec_fetch_str_str(kvv, "hostgroup.notes_url"),
			kvvec_fetch_str_str(kvv, "hostgroup.action_url"));
		objs->hg  = temp_create_hostgroup(kvv, hg);
	}
	tmp = kvvec_fetch_str_str(kvv, "servicegroup.group_name");
	if (tmp) {
		servicegroup *sg = create_servicegroup(
			tmp,
			kvvec_fetch_str_str(kvv, "servicegroup.alias"),
			kvvec_fetch_str_str(kvv, "servicegroup.notes"),
			kvvec_fetch_str_str(kvv, "servicegroup.notes_url"),
			kvvec_fetch_str_str(kvv, "servicegroup.action_url"));
		objs->sg  = temp_create_servicegroup(kvv, sg);
	}
	tmp = kvvec_fetch_str_str(kvv, "command.name");
	if (tmp) {
		command *cmd = create_command(kvvec_fetch_str_str(kvv, "command.name"), kvvec_fetch_str_str(kvv, "command.command_line"));
		objs->cmd = temp_create_command(kvv, cmd);
	}

	/* Set up relations */
	if( objs->hst && objs->hg ) {
		add_host_to_hostgroup(objs->hg, objs->hst);
	}
	if( objs->svc && objs->sg ) {
		objs->sg->members = calloc(1,sizeof(servicesmember));
		if( objs->sg->members ) {
			objs->sg->members->host_name = objs->svc->host_name;
			objs->sg->members->service_description = objs->svc->description;
			objs->sg->members->service_ptr = objs->svc;
		}
		objs->svc->servicegroups_ptr = calloc(1,sizeof(objectlist));
		if( objs->svc->servicegroups_ptr ) {
			objs->svc->servicegroups_ptr->object_ptr = (void*)objs->sg;
		}
	}
	if( objs->ctc && objs->cg ) {
		objs->cg->members = calloc(1,sizeof(contactsmember));
		if( objs->cg->members ) {
			objs->cg->members->contact_name = objs->ctc->name;
			objs->cg->members->contact_ptr = objs->ctc;
		}
		objs->ctc->contactgroups_ptr = calloc(1,sizeof(objectlist));
		if( objs->ctc->contactgroups_ptr ) {
			objs->ctc->contactgroups_ptr->object_ptr = (void*)objs->cg;
		}
	}
	objs->runcommand = kvvec_fetch_str_str(kvv,"runcmd");

	if( objs->svc ) {
		objs->timeout = service_check_timeout;
	} else {
		objs->timeout = host_check_timeout;
	}
	tmp = kvvec_fetch_str_str(kvv,"timeout");
	if( tmp ) objs->timeout = atoi( tmp );

	return objs;
}
void destroy_temp_objects( struct temp_objects *objs ) {
	destroy_contact(objs->ctc);
	destroy_contactgroup(objs->cg);
	destroy_hostgroup(objs->hg);
	destroy_host(objs->hst);
	destroy_servicegroup(objs->sg);
	destroy_service(objs->svc);
	destroy_command(objs->cmd);
	/* Don't free runcommand, that's referenced from kvv */
	free(objs);
}
